@@ -4,7 +4,7 @@
     created              : Sat Mar 18 23:16:38 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: human.cpp,v 1.39 2004/02/08 19:20:33 torcs Exp $
+    version              : $Id: human.cpp,v 1.41 2005/02/01 18:33:36 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file   
     		
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: human.cpp,v 1.39 2004/02/08 19:20:33 torcs Exp $
+    @version	$Id: human.cpp,v 1.41 2005/02/01 18:33:36 berniw Exp $
 */
 
 
@@ -60,7 +60,7 @@ static char	sstring[1024];
 static char	buf[1024];
 
 static tTrack	*curTrack;
-static void	*DrvInfo;
+//static void	*DrvInfo;
 
 static float color[] = {0.0, 0.0, 1.0, 1.0};
 
@@ -74,7 +74,7 @@ static int speedLimiter	= 0;
 static tdble Vtarget;
 
 
-typedef struct 
+typedef struct
 {
     int		state;
     int		edgeDn;
@@ -87,7 +87,7 @@ static tKeyInfo skeyInfo[256];
 static int currentKey[256];
 static int currentSKey[256];
 
-
+static int	firstTime = 0;
 
 #ifdef _WIN32
 /* should be present in mswindows */
@@ -100,20 +100,20 @@ BOOL WINAPI DllEntryPoint (HINSTANCE hDLL, DWORD dwReason, LPVOID Reserved)
 static void
 shutdown(int index)
 {
-    static int	firstTime = 1;
-    int		idx = index - 1;
+	//static int	firstTime = 1;
+	int		idx = index - 1;
 
-    free (HCtx[idx]);
+	free (HCtx[idx]);
 
-    if (firstTime) {
-	GfParmReleaseHandle(DrvInfo);
-	GfParmReleaseHandle(PrefHdle);
-	GfctrlJoyRelease(joyInfo);
-	GfctrlMouseRelease(mouseInfo);
-	GfuiKeyEventRegisterCurrent(NULL);
-	GfuiSKeyEventRegisterCurrent(NULL);
-	firstTime = 0;
-    }
+	if (firstTime) {
+		//GfParmReleaseHandle(DrvInfo);
+		GfParmReleaseHandle(PrefHdle);
+		GfctrlJoyRelease(joyInfo);
+		GfctrlMouseRelease(mouseInfo);
+		GfuiKeyEventRegisterCurrent(NULL);
+		GfuiSKeyEventRegisterCurrent(NULL);
+		firstTime = 0;
+	}
 }
 
 
@@ -132,7 +132,7 @@ shutdown(int index)
  *	0
  *
  * Remarks
- *	
+ *
  */
 static int
 InitFuncPt(int index, void *pt)
@@ -144,7 +144,20 @@ InitFuncPt(int index, void *pt)
 	masterPlayer = index;
     }
 
-    /* Allocate a new context for that player */
+	if (firstTime < 1) {
+		firstTime = 1;
+		//DrvInfo = GfParmReadFile(buf, GFPARM_RMODE_REREAD | GFPARM_RMODE_CREAT);
+		joyInfo = GfctrlJoyInit();
+		if (joyInfo) {
+			joyPresent = 1;
+		}
+
+		mouseInfo = GfctrlMouseInit();
+
+	}
+
+
+	/* Allocate a new context for that player */
     HCtx[idx] = (tHumanContext *) calloc (1, sizeof (tHumanContext));
 
     HCtx[idx]->ABS = 1.0;
@@ -182,49 +195,51 @@ InitFuncPt(int index, void *pt)
  *	0
  *
  * Remarks
- *	
+ *
  */
+
+#define MAXNAMELEN 100
+
+static char names[10][MAXNAMELEN];
+
 extern "C" int
 human(tModInfo *modInfo)
 {
-    int		i;
-    char	*driver;
+	int i;
+	char *driver;
 
-    memset(modInfo, 0, 10*sizeof(tModInfo));
+	memset(modInfo, 0, 10*sizeof(tModInfo));
 
-    sprintf(buf, "%sdrivers/human/human.xml", GetLocalDir());
-    DrvInfo = GfParmReadFile(buf, GFPARM_RMODE_REREAD | GFPARM_RMODE_CREAT);
-    
-    if (DrvInfo != NULL) {
-	for (i = 0; i < 10; i++) {
-	    sprintf(sstring, "Robots/index/%d", i+1);
-	    driver = GfParmGetStr(DrvInfo, sstring, "name", "");
-	    if (strlen(driver) == 0) {
-		break;
-	    }
-	    modInfo->name    = driver;	/* name of the module (short) */
-	    modInfo->desc    = "Joystick controlable driver";	/* description of the module (can be long) */
-	    modInfo->fctInit = InitFuncPt;	/* init function */
-	    modInfo->gfId    = ROB_IDENT;	/* supported framework version */
-	    modInfo->index   = i+1;
-	    modInfo++;
+	sprintf(buf, "%sdrivers/human/human.xml", GetLocalDir());
+	void *DrvInfo = GfParmReadFile(buf, GFPARM_RMODE_REREAD | GFPARM_RMODE_CREAT);
+
+	if (DrvInfo != NULL) {
+		for (i = 0; i < 10; i++) {
+			sprintf(sstring, "Robots/index/%d", i+1);
+			driver = GfParmGetStr(DrvInfo, sstring, "name", "");
+			if (strlen(driver) == 0) {
+				break;
+			}
+			strncpy(names[i], driver, MAXNAMELEN);
+			modInfo->name    = names[i];	/* name of the module (short) */
+			//modInfo->name    = driver;	/* name of the module (short) */
+			modInfo->desc    = "Joystick controlable driver";	/* description of the module (can be long) */
+			modInfo->fctInit = InitFuncPt;	/* init function */
+			modInfo->gfId    = ROB_IDENT;	/* supported framework version */
+			modInfo->index   = i+1;
+			modInfo++;
+		}
+		// Just release in case we got it.
+		GfParmReleaseHandle(DrvInfo);
 	}
-    }
-    
-    joyInfo = GfctrlJoyInit();
-    if (joyInfo) {
-	joyPresent = 1;
-    }
 
-    mouseInfo = GfctrlMouseInit();
-
-    return 0;
+	return 0;
 }
 
 
 /*
  * Function
- *	
+ *
  *
  * Description
  *	search under drivers/human/tracks/<trackname>/car-<model>-<index>.xml
@@ -233,74 +248,85 @@ human(tModInfo *modInfo)
  *		     drivers/human/car-<model>.xml
  *
  * Parameters
- *	
+ *
  *
  * Return
- *	
+ *
  *
  * Remarks
- *	
+ *
  */
 static void initTrack(int index, tTrack* track, void *carHandle, void **carParmHandle, tSituation *s)
 {
-    char	*carname;
-    char	*s1, *s2;
-    char	trackname[256];
-    tdble	fuel;
-    int		idx = index - 1;
+	char	*carname;
+	char	*s1, *s2;
+	char	trackname[256];
+	tdble	fuel;
+	int		idx = index - 1;
 
-    curTrack = track;
-    s1 = strrchr(track->filename, '/') + 1;
-    s2 = strchr(s1, '.');
-    strncpy(trackname, s1, s2-s1);
-    trackname[s2-s1] = 0;
-    sprintf(sstring, "Robots/index/%d", index);
-    carname = GfParmGetStr(DrvInfo, sstring, "car name", "");
-    sprintf(sstring, "%sdrivers/human/tracks/%s/car-%s-%d.xml", GetLocalDir(), trackname, carname, index);
-    *carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
-    if (*carParmHandle != NULL) {
-	GfOut("Player: %s Loaded\n", sstring);
-    } else {
-	sprintf(sstring, "%sdrivers/human/tracks/%s/car-%s.xml", GetLocalDir(), trackname, carname);
+	curTrack = track;
+	s1 = strrchr(track->filename, '/') + 1;
+	s2 = strchr(s1, '.');
+	strncpy(trackname, s1, s2-s1);
+	trackname[s2-s1] = 0;
+	sprintf(sstring, "Robots/index/%d", index);
+
+	sprintf(buf, "%sdrivers/human/human.xml", GetLocalDir());
+	void *DrvInfo = GfParmReadFile(buf, GFPARM_RMODE_REREAD | GFPARM_RMODE_CREAT);
+	carname = "";
+	if (DrvInfo != NULL) {
+		carname = GfParmGetStr(DrvInfo, sstring, "car name", "");
+	}
+
+	sprintf(sstring, "%sdrivers/human/tracks/%s/car-%s-%d.xml", GetLocalDir(), trackname, carname, index);
 	*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
 	if (*carParmHandle != NULL) {
-	    GfOut("Player: %s Loaded\n", sstring);
-	} else {
-	    sprintf(sstring, "%sdrivers/human/car-%s-%d.xml", GetLocalDir(), carname, index);
-	    *carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
-	    if (*carParmHandle != NULL) {
 		GfOut("Player: %s Loaded\n", sstring);
-	    } else {
-		sprintf(sstring, "%sdrivers/human/car-%s.xml", GetLocalDir(), carname);
+	} else {
+		sprintf(sstring, "%sdrivers/human/tracks/%s/car-%s.xml", GetLocalDir(), trackname, carname);
 		*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
 		if (*carParmHandle != NULL) {
-		    GfOut("Player: %s Loaded\n", sstring);
-		} else {
-		    sprintf(sstring, "%sdrivers/human/car.xml", GetLocalDir ());
-		    *carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
-		    if (*carParmHandle != NULL) {
 			GfOut("Player: %s Loaded\n", sstring);
-		    }
+		} else {
+			sprintf(sstring, "%sdrivers/human/car-%s-%d.xml", GetLocalDir(), carname, index);
+			*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
+			if (*carParmHandle != NULL) {
+				GfOut("Player: %s Loaded\n", sstring);
+			} else {
+				sprintf(sstring, "%sdrivers/human/car-%s.xml", GetLocalDir(), carname);
+				*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
+				if (*carParmHandle != NULL) {
+					GfOut("Player: %s Loaded\n", sstring);
+				} else {
+					sprintf(sstring, "%sdrivers/human/car.xml", GetLocalDir ());
+					*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
+					if (*carParmHandle != NULL) {
+						GfOut("Player: %s Loaded\n", sstring);
+					}
+				}
+			}
 		}
-	    }
 	}
-    }
-    if (curTrack->pits.type != TR_PIT_NONE) {
-	sprintf(sstring, "%s/%s/%d", HM_SECT_PREF, HM_LIST_DRV, index);
-	HCtx[idx]->NbPitStopProg = (int)GfParmGetNum(PrefHdle, sstring, HM_ATT_NBPITS, (char*)NULL, 0);
-	GfOut("Player: index %d , Pits stops %d\n", index, HCtx[idx]->NbPitStopProg);
-    } else {
-	HCtx[idx]->NbPitStopProg = 0;
-    }
-    fuel = 0.0008 * curTrack->length * (s->_totLaps + 1) / (1.0 + ((tdble)HCtx[idx]->NbPitStopProg)) + 20.0;
-    GfParmSetNum(*carParmHandle, SECT_CAR, PRM_FUEL, (char*)NULL, fuel);
-    Vtarget = curTrack->pits.speedLimit;
+
+	if (curTrack->pits.type != TR_PIT_NONE) {
+		sprintf(sstring, "%s/%s/%d", HM_SECT_PREF, HM_LIST_DRV, index);
+		HCtx[idx]->NbPitStopProg = (int)GfParmGetNum(PrefHdle, sstring, HM_ATT_NBPITS, (char*)NULL, 0);
+		GfOut("Player: index %d , Pits stops %d\n", index, HCtx[idx]->NbPitStopProg);
+	} else {
+		HCtx[idx]->NbPitStopProg = 0;
+	}
+	fuel = 0.0008 * curTrack->length * (s->_totLaps + 1) / (1.0 + ((tdble)HCtx[idx]->NbPitStopProg)) + 20.0;
+	GfParmSetNum(*carParmHandle, SECT_CAR, PRM_FUEL, (char*)NULL, fuel);
+	Vtarget = curTrack->pits.speedLimit;
+	if (DrvInfo != NULL) {
+		GfParmReleaseHandle(DrvInfo);
+	}
 }
 
 
 /*
  * Function
- *	
+ *
  *
  * Description
  *	
@@ -794,13 +820,15 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
     if (speedLimiter) {
 	tdble	Dv;
 
-	Dv = Vtarget - car->_speed_x;
+	if (Vtarget != 0) {
+	    Dv = Vtarget - car->_speed_x;
 
-	if (Dv > 0.0) {
-	    car->_accelCmd = MIN(car->_accelCmd, fabs(Dv/6.0));
-	} else {
-	    car->_brakeCmd = MAX(car->_brakeCmd, fabs(Dv/5.0));
-	    car->_accelCmd = 0;
+	    if (Dv > 0.0) {
+		car->_accelCmd = MIN(car->_accelCmd, fabs(Dv/6.0));
+	    } else {
+		car->_brakeCmd = MAX(car->_brakeCmd, fabs(Dv/5.0));
+		car->_accelCmd = 0;
+	    }
 	}
     }
     
