@@ -4,7 +4,7 @@
     created              : Sun Jan 30 12:00:15 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: car.h,v 1.27 2004/02/06 23:08:53 torcs Exp $
+    version              : $Id: car.h,v 1.32 2004/11/28 18:53:30 olethros Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file
     		This is the car structure.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: car.h,v 1.27 2004/02/06 23:08:53 torcs Exp $
+    @version	$Id: car.h,v 1.32 2004/11/28 18:53:30 olethros Exp $
     @ingroup	carstruct
     @note	Short cuts are to be used with the carElt structure.
 */
@@ -245,14 +245,37 @@ typedef struct {
     int		state;		/**< wheel state */
     tTrackSeg	*seg;		/**< Track segment where the wheel is */
     tdble rollRes;              /**< rolling resistance, useful for sound */
+	tdble   temp_in, temp_mid, temp_out;
+	tdble   condition;
+	tdble slipSide;
+	tdble slipAccel;
+	tdble Fx;
+	tdble Fy;
+	tdble Fz;
 } tWheelState;
 #define _ride(i)	priv.wheel[i].relPos.z
 #define _brakeTemp(i)	priv.wheel[i].brakeTemp
 #define _wheelSpinVel(i) priv.wheel[i].spinVel
 #define _wheelSeg(i)	priv.wheel[i].seg
+#define _wheelSlipSide(i) priv.wheel[i].slipSide
+#define _wheelSlipAccel(i) priv.wheel[i].slipAccel
+#define _wheelFx(i) priv.wheel[i].Fx
+#define _wheelFy(i) priv.wheel[i].Fy
+#define _wheelFz(i) priv.wheel[i].Fz
+#define _tyreT_in(i) priv.wheel[i].temp_in
+#define _tyreT_mid(i) priv.wheel[i].temp_mid
+#define _tyreT_out(i) priv.wheel[i].temp_out
+#define _tyreCondition(i) priv.wheel[i].condition
+
 
 #define MAX_GEARS	10	/* including reverse and neutral */
 
+typedef struct tCollisionState_ {
+	int collision_count;
+	sgVec3 pos;
+	sgVec3 force;
+} tCollisionState;
+
 /** Data known only by the driver */
 typedef struct {
     void	*paramsHandle;	/**< accessible parameters for modules */
@@ -263,22 +286,31 @@ typedef struct {
     tPosd	corner[4];	/**< car's corners position */
     int		gear;	    	/**< current gear */
     tdble	fuel;	    	/**< remaining fuel (liters) */
+	tdble   fuel_consumption_total; // l
+	tdble   fuel_consumption_instant; // l/100km (>100 means infinity)
     tdble	enginerpm;
     tdble	enginerpmRedLine;
     tdble	enginerpmMax;
     tdble	enginerpmMaxTq;
+    tdble	enginerpmMaxPw;
+    tdble	engineMaxTq;
+    tdble	engineMaxPw;
     tdble	gearRatio[MAX_GEARS];	/**< including final drive */
     int		gearNb;			/**< incl reverse and neutral */
     int		gearOffset;		/**< gearRatio[gear + gearOffset] is the ratio for gear */
     tdble	skid[4];		/**< skid intensity */
-    tdble	reaction[4];
+    tdble	reaction[4];    /**< reaction on wheels */
     int		collision;
+	float   smoke;
     t3Dd	normal;
-    t3Dd	collpos;
+    t3Dd	collpos;        /**< Collision position, useful for sound*/
     int		dammage;
     int		debug;
+	tCollisionState collision_state; /**< collision state */
 } tPrivCar;
 /* structure access */
+#define _fuelTotal priv.fuel_consumption_total
+#define _fuelInstant priv.fuel_consumption_instant
 #define _driverIndex	priv.driverIndex
 #define _paramsHandle	priv.paramsHandle
 #define _carHandle	priv.carHandle
@@ -287,6 +319,9 @@ typedef struct {
 #define _enginerpmRedLine	priv.enginerpmRedLine
 #define _enginerpmMax	priv.enginerpmMax
 #define _enginerpmMaxTq	priv.enginerpmMaxTq
+#define _enginerpmMaxPw	priv.enginerpmMaxPw
+#define _engineMaxTq	priv.engineMaxTq
+#define _engineMaxPw	priv.engineMaxPw
 #define _gearRatio	priv.gearRatio
 #define _gearNb		priv.gearNb
 #define _gearOffset	priv.gearOffset
@@ -360,6 +395,7 @@ typedef struct CarElt
 
 /* sections in xml description files */
 
+#define SECT_SIMU_SETTINGS "Simulation Options"
 #define SECT_CAR		"Car"
 #define SECT_FRNT		"Front"
 #define SECT_FRNTWING		"Front Wing"
@@ -542,8 +578,7 @@ typedef struct CarElt
 #define PRM_SPEEDO_MAXVAL	"speedometer max value"
 #define PRM_SPEEDO_MINANG	"speedometer min angle"
 #define PRM_SPEEDO_MAXANG	"speedometer max angle"
-#define PRM_SPEEDO_DIGITAL	"speedometer digital"
-
+#define PRM_SPEEDO_DIGITAL  "speedometer digital"
 /* Lights */
 #define	VAL_LIGHT_HEAD1		"head1"
 #define	VAL_LIGHT_HEAD2		"head2"
@@ -551,7 +586,12 @@ typedef struct CarElt
 #define	VAL_LIGHT_BRAKE2	"brake2"
 #define	VAL_LIGHT_REVERSE	"reverse"
 #define	VAL_LIGHT_REAR		"rear"
-
+/* Simulation Options */
+#define PRM_DAMAGE_TYRES "damage/tyres"
+#define PRM_DAMAGE_SUSPENSION "damage/suspension"
+#define PRM_DAMAGE_ALIGNMENT "damage/alignment"
+#define PRM_DAMAGE_AERO "damage/aero"
+#define PRM_MODEL_AEROFLOW "model/aeroflow"
 
 #endif /* __CARV1_H__ */ 
 
