@@ -2,9 +2,9 @@
 
     file        : raceengine.cpp
     created     : Sat Nov 23 09:05:23 CET 2002
-    copyright   : (C) 2002 by Eric Espi                        
+    copyright   : (C) 2002 by Eric Espié                        
     email       : eric.espie@torcs.org   
-    version     : $Id: raceengine.cpp,v 1.7 2004/02/08 19:20:33 torcs Exp $                                  
+    version     : $Id: raceengine.cpp,v 1.13 2005/02/01 15:55:51 berniw Exp $                                  
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file   
     		
     @author	<a href=mailto:eric.espie@torcs.org>Eric Espie</a>
-    @version	$Id: raceengine.cpp,v 1.7 2004/02/08 19:20:33 torcs Exp $
+    @version	$Id: raceengine.cpp,v 1.13 2005/02/01 15:55:51 berniw Exp $
 */
 
 #include <stdlib.h>
@@ -53,12 +53,19 @@ ReUpdtPitTime(tCarElt *car)
 {
     tSituation	*s = ReInfo->s;
     tReCarInfo	*info = &(ReInfo->_reCarInfo[car->index]);
+    int		i;
 
     switch (car->_pitStopType) {
     case RM_PIT_REPAIR:
 	info->totalPitTime = 2.0 + fabs(car->_pitFuel) / 8.0 + (tdble)(fabs(car->_pitRepair)) * 0.007;
 	car->_scheduledEventTime = s->currentTime + info->totalPitTime;
 	ReInfo->_reSimItf.reconfig(car);
+	for (i=0; i<4; i++) {
+		car->_tyreCondition(i) = 1.01;
+		car->_tyreT_in(i) = 50.0;
+		car->_tyreT_mid(i) = 50.0;
+		car->_tyreT_out(i) = 50.0;
+	}
 	break;
     case RM_PIT_STOPANDGO:
 	info->totalPitTime = 0.0;
@@ -139,7 +146,7 @@ ReManage(tCarElt *car)
 	}
     } else if ((car->_pit) && (car->ctrl.raceCmd & RM_CMD_PIT_ASKED)) {
 	tdble lgFromStart = car->_trkPos.seg->lgfromstart;
-	
+
 	switch (car->_trkPos.seg->type) {
 	case TR_STR:
 	    lgFromStart += car->_trkPos.toStart;
@@ -193,7 +200,7 @@ ReManage(tCarElt *car)
 	    }
 	}
     }
-    
+
     /* Start Line Crossing */
     if (info->prevTrkPos.seg != car->_trkPos.seg) {
 	if ((info->prevTrkPos.seg->raceInfo & TR_LAST) && (car->_trkPos.seg->raceInfo & TR_START)) {
@@ -225,10 +232,15 @@ ReManage(tCarElt *car)
 			case RM_TYPE_PRACTICE:
 			    if (ReInfo->_displayMode == RM_DISP_MODE_NONE) {
 				ReInfo->_refreshDisplay = 1;
-				sprintf(buf,"lap: %02d   time: %s  best: %s  top spd: %.2f    min spd: %.2f    dammage: %d", 
-					car->_laps - 1, GfTime2Str(car->_lastLapTime, 0), GfTime2Str(car->_bestLapTime, 0),
+				char *t1, *t2;
+				t1 = GfTime2Str(car->_lastLapTime, 0);
+				t2 = GfTime2Str(car->_bestLapTime, 0);
+				sprintf(buf,"lap: %02d   time: %s  best: %s  top spd: %.2f    min spd: %.2f    damage: %d",
+					car->_laps - 1, t1, t2,
 					info->topSpd * 3.6, info->botSpd * 3.6, car->_dammage);
 				ReResScreenAddText(buf);
+				free(t1);
+				free(t2);
 			    }
 			    /* save the lap result */
 			    ReSavePracticeLap(car);
@@ -256,20 +268,23 @@ ReManage(tCarElt *car)
 				sprintf(buf, "Winner %s", car->_name);
 				ReRaceBigMsgSet(buf, 10);
 			    } else {
-				switch (car->_pos % 10) {
-				case 1:
-				    sprintf(buf, "%s Finished %dst", car->_name, car->_pos);
-				    break;
-				case 2:
-				    sprintf(buf, "%s Finished %dnd", car->_name, car->_pos);
-				    break;
-				case 3:
-				    sprintf(buf, "%s Finished %drd", car->_name, car->_pos);
-				    break;
-				default:
-				    sprintf(buf, "%s Finished %dth", car->_name, car->_pos);
-				    break;
-				}
+ 				char *numSuffix = "th";
+ 				if (abs(12 - car->_pos) > 1) { /* leave suffix as 'th' for 11 to 13 */
+				    switch (car->_pos % 10) {
+				    case 1:
+					numSuffix = "st";
+					break;
+				    case 2:
+					numSuffix = "nd";
+					break;
+				    case 3:
+					numSuffix = "rd";
+					break;
+				    default:
+					break;
+				    }
+  				}
+ 				sprintf(buf, "%s Finished %d%s", car->_name, car->_pos, numSuffix);
 				ReRaceMsgSet(buf, 5);
 			    }
 			}
@@ -349,12 +364,25 @@ ReRaceRules(tCarElt *car)
     tTrackSeg		*prevSeg = RtTrackGetSeg(&(info->prevTrkPos));
     static float	color[] = {0.0, 0.0, 1.0, 1.0};
 
-    if (car->_skillLevel < 3) {
+	// DNF cars which need too much time for the current lap, this is mainly to avoid
+	// that a "hanging" driver can stop the quali from finishing.
+	// Allowed time is longest pitstop possible + time for tracklength with speed??? (currently fixed 10 [m/s]).
+	// for simplicity. Human driver is an exception to this rule, to allow explorers
+	// to enjoy the landscape.
+	// TODO: Make it configurable.
+	if ((car->_curLapTime > 84.5 + ReInfo->track->length/10.0) &&
+		(car->_driverType != RM_DRV_HUMAN))
+	{
+		car->_state |= RM_CAR_STATE_ELIMINATED;
+	    return;
+	}
+
+	if (car->_skillLevel < 3) {
 	/* only for the pros */
 	return;
     }
-    
-    penalty = GF_TAILQ_FIRST(&(car->_penaltyList));
+
+	penalty = GF_TAILQ_FIRST(&(car->_penaltyList));
     if (penalty) {
 	if (car->_laps > penalty->lapToClear) {
 	    /* too late to clear the penalty, out of race */
@@ -476,11 +504,11 @@ ReOneStep(double deltaTimeIncrement)
     tSituation	*s = ReInfo->s;
 
     if (floor(s->currentTime) == -2.0) {
-	ReRaceBigMsgSet("Ready !", 1.0);
+	ReRaceBigMsgSet("Ready", 1.0);
     } else if (floor(s->currentTime) == -1.0) {
-	ReRaceBigMsgSet("Set !", 1.0);
+	ReRaceBigMsgSet("Set", 1.0);
     } else if (floor(s->currentTime) == 0.0) {
-	ReRaceBigMsgSet("Go !", 1.0);
+	ReRaceBigMsgSet("Go", 1.0);
     }
 
     ReInfo->_reCurTime += deltaTimeIncrement * ReInfo->_reTimeMult; /* "Real" time */
@@ -550,7 +578,7 @@ reCapture(void)
     glReadBuffer(GL_FRONT);
     glReadPixels((sw-vw)/2, (sh-vh)/2, vw, vh, GL_RGB, GL_UNSIGNED_BYTE, (GLvoid*)img);
 
-    sprintf(buf, "%s/torcs-%04.4d-%08.8d.png", capture->outputBase, capture->currentCapture, capture->currentFrame++);
+    sprintf(buf, "%s/torcs-%4.4d-%8.8d.png", capture->outputBase, capture->currentCapture, capture->currentFrame++);
     GfImgWritePng(img, buf, vw, vh);
     free(img);
 }
