@@ -4,7 +4,7 @@
     created              : Sat Aug 14 23:03:22 CEST 1999
     copyright            : (C) 1999 by Eric Espie                         
     email                : torcs@free.fr   
-    version              : $Id: rttrack.cpp,v 1.14 2003/11/23 20:21:12 torcs Exp $                                  
+    version              : $Id: rttrack.cpp,v 1.17 2005/02/01 15:55:53 berniw Exp $                                  
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,7 +19,7 @@
 /** @file	
     		This is a collection of useful functions for programming a robot.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: rttrack.cpp,v 1.14 2003/11/23 20:21:12 torcs Exp $
+    @version	$Id: rttrack.cpp,v 1.17 2005/02/01 15:55:53 berniw Exp $
     @ingroup	robottools
 */
 
@@ -426,7 +426,7 @@ RtTrackHeightL(tTrkLocPos *p)
     if (seg->style == TR_CURB) {
 	if (seg->type2 == TR_RBORDER) {
 	    return seg->vertex[TR_SR].z + p->toStart * seg->Kzl +
-		tr * tan(seg->angle[TR_XS] + p->toStart * seg->Kzw + atan2(seg->height, seg->width)) +
+		(seg->width - tr) * tan(seg->angle[TR_XS] + p->toStart * seg->Kzw + atan2(seg->height, seg->width)) +
 		seg->surface->kRoughness * sin(seg->surface->kRoughWaveLen * lg) * (seg->width - tr) / seg->width;
 	}
 	
