@@ -4,7 +4,7 @@
     created              : Fri Aug 13 22:32:14 CEST 1999
     copyright            : (C) 1999 by Eric Espie                         
     email                : torcs@free.fr   
-    version              : $Id: tgf.h,v 1.37 2003/08/23 23:43:14 torcs Exp $                                  
+    version              : $Id: tgf.h,v 1.40 2005/02/01 15:55:54 berniw Exp $                                  
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,7 +19,7 @@
 /** @file   
     	The Gaming Framework API.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: tgf.h,v 1.37 2003/08/23 23:43:14 torcs Exp $
+    @version	$Id: tgf.h,v 1.40 2005/02/01 15:55:54 berniw Exp $
 */
 
 
@@ -68,7 +68,7 @@ extern void GfInit(void);
 #define freez FREEZ
 
 const double PI = 3.14159265358979323846;  /**< PI */
-const tdble G = 9.80665; /**< m/s/s */
+const tdble G = 9.80665f; /**< m/s/s */
 
 /* conversion */
 #define RADS2RPM(x) ((x)*9.549296585)		/**< Radian/s to RPM conversion */
@@ -240,8 +240,9 @@ typedef struct FList
 } tFList;
 
 extern tFList *GfDirGetList(char *dir);
+extern tFList *GfDirGetListFiltered(char *dir, char *suffix);
 typedef void (*tfDirfreeUserData)(void*);	/**< Function to call for releasing the user data associated with file entry */
-extern void GfDirFreeList(tFList *list, tfDirfreeUserData freeUserData);
+extern void GfDirFreeList(tFList *list, tfDirfreeUserData freeUserDatabool, bool freename = false, bool freedispname = false);
 
 
 /**********************************
@@ -277,6 +278,9 @@ extern int GfParmWriteFile(const char *file, void* handle, char *name);
 extern char *GfParmGetName(void *handle);
 extern char *GfParmGetFileName(void *handle);
 
+/* set the dtd and header values */
+extern void GfParmSetDTD (void *parmHandle, char *dtd, char*header);
+
 /* get string parameter value */
 extern char *GfParmGetStr(void *handle, char *path, char *key, char *deflt);
 /* get string parameter value */
