@@ -2,9 +2,9 @@
                            screen.cpp -- screen init
                              -------------------
     created              : Fri Aug 13 22:29:56 CEST 1999
-    copyright            : (C) 1999, 2002 by Eric Espie
+    copyright            : (C) 1999, 2004 by Eric Espie, Bernhard Wymann
     email                : torcs@free.fr
-    version              : $Id: screen.cpp,v 1.10 2003/12/15 09:37:50 berniw Exp $
+    version              : $Id: screen.cpp,v 1.15 2005/02/01 15:55:55 berniw Exp $
 ***************************************************************************/
 
 /***************************************************************************
@@ -19,7 +19,7 @@
 /** @file
     Screen management.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: screen.cpp,v 1.10 2003/12/15 09:37:50 berniw Exp $
+    @version	$Id: screen.cpp,v 1.15 2005/02/01 15:55:55 berniw Exp $
     @ingroup	screen
 */
 
@@ -40,6 +40,18 @@
 #include "gui.h"
 #include "fg_gm.h"
 
+//#ifndef WIN32
+//#define USE_RANDR_EXT
+//#endif // WIN32
+
+#ifdef USE_RANDR_EXT
+#include <GL/glx.h>
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#include <X11/keysym.h>
+#include <X11/extensions/Xrandr.h>
+#endif // USE_RANDR_EXT
+
 static int GfScrWidth;
 static int GfScrHeight;
 static int GfViewWidth;
@@ -55,26 +67,37 @@ static int usedGM = 0;
 static int usedFG = 0;
 #endif
 
-static char	*Res[] = {"640x480", "800x600", "1024x768", "1200x960", "1280x1024", "1600x1200", "320x200"};
-static char	*Mode[] = {"Full-screen mode", "Window mode"};
-static char	*Depth[] = {"24", "32", "8", "16"};
-
+#ifdef USE_RANDR_EXT
+static char	**Res = NULL;
+static int nbRes = 0;
+#else // USE_RANDR_EXT
+static char	*Res[] = {"640x480", "800x600", "1024x768", "1152x864", "1200x960", "1280x1024", "1600x1200", "320x200"};
 static const int nbRes = sizeof(Res) / sizeof(Res[0]);
+#endif // USE_RANDR_EXT
+
+static char	*Mode[] = {"Full-screen mode", "Window mode"};
+static char *VInit[] = {GFSCR_VAL_VINIT_COMPATIBLE, GFSCR_VAL_VINIT_BEST};
+static char	*Depthlist[] = {"24", "32", "16"};
+
+//static const int nbRes = sizeof(Res) / sizeof(Res[0]);
 static const int nbMode = sizeof(Mode) / sizeof(Mode[0]);
-static const int nbDepth = sizeof(Depth) / sizeof(Depth[0]);
+static const int nbVInit = sizeof(VInit) / sizeof(VInit[0]);
+static const int nbDepth = sizeof(Depthlist) / sizeof(Depthlist[0]);
 
 static int	curRes = 0;
 static int	curMode = 0;
 static int	curDepth = 0;
+static int curVInit = 0;
 
-static int	curMaxFreq = 160;
-#if WIN32
+static int	curMaxFreq = 75;
+#ifdef WIN32
 static int	MaxFreqId;
 #endif
 
 static int	ResLabelId;
 static int	DepthLabelId;
 static int	ModeLabelId;
+static int VInitLabelId;
 
 static void	*paramHdle;
 
@@ -84,6 +107,116 @@ static float LabelColor[] = {1.0, 0.0, 1.0, 1.0};
 void
 gfScreenInit(void)
 {
+#ifdef USE_RANDR_EXT
+	// Get display, screen and root window handles.
+	char *displayname = getenv("DISPLAY");
+	if (displayname == NULL) {
+		displayname = ":0.0";
+	}
+
+	Display *display = XOpenDisplay(displayname);
+
+	if( display != NULL) {
+		// If we have a display fill in the resolutions advertised by Xrandr.
+		int screen = DefaultScreen(display);
+    	Window root = RootWindow(display, screen);
+
+		XRRScreenConfiguration *screenconfig = XRRGetScreenInfo (display, root);
+		if (screenconfig != NULL) {
+			int i, j, nsize;
+			XRRScreenSize *sizes = XRRConfigSizes(screenconfig, &nsize);
+
+			if (nsize > 0) {
+				// Check if 320x200, 640x480, 800x600 are available, construct a mode wish list.
+				int check_resx[] = {320, 640, 800};
+				int check_resy[] = {240, 480, 600};
+				bool mode_in_list[] = {false, false, false};
+				int add_modes = sizeof(check_resx)/sizeof(check_resx[0]);
+
+				for (i = 0; i < nsize; i++) {
+					for (j = 0; j < 3; j++) {
+						if ((mode_in_list[j] == false) && (sizes[i].width == check_resx[j])) {
+							if (sizes[i].height == check_resy[j]) {
+								// Mode already in list.
+								mode_in_list[j] = true;
+								add_modes--;
+							}
+						}
+					}
+				}
+
+				const int bufsize = 20;
+				char buffer[bufsize];
+				Res = (char**) malloc(sizeof(char *)*(nsize+add_modes));
+				int resx[nsize+add_modes];
+				int resy[nsize+add_modes];
+				for (i = 0; i < nsize+add_modes; i++) {
+					if (i < nsize) {
+						// Add mode from screenconfig (system).
+						snprintf(buffer, bufsize, "%dx%d", sizes[i].width, sizes[i].height);
+						Res[i] = strndup(buffer, bufsize);
+						resx[i] = sizes[i].width;
+						resy[i] = sizes[i].height;
+					} else {
+						// Add mode from wish list.
+						unsigned int j;
+						for (j = 0; j < sizeof(check_resx)/sizeof(check_resx[0]); j++) {
+							if (mode_in_list[j] == false) {
+								mode_in_list[j] = true;
+								snprintf(buffer, bufsize, "%dx%d", check_resx[j], check_resy[j]);
+								Res[i] = strndup(buffer, bufsize);
+								resx[i] = check_resx[j];
+								resy[i] = check_resy[j];
+								break;
+							}
+						}
+					}
+
+					// Stupid sorting (not much elements, don't worry).
+					int j;
+					for (j = i; j > 0; j--) {
+						if ((resx[j] < resx[j-1]) ||
+							(resx[j] == resx[j-1] && resy[j] < resy[j-1]))
+						{
+							int tx, ty;
+							char *tc;
+							tx = resx[j-1];
+							ty = resy[j-1];
+							resx[j-1] = resx[j];
+							resy[j-1] = resy[j];
+							resx[j] = tx;
+							resy[j] = ty;
+							tc = Res[j-1];
+							Res[j-1] = Res[j];
+							Res[j] = tc;
+						} else {
+							break;
+						}
+					}
+				}
+			}
+
+			nbRes = nsize;
+			XRRFreeScreenConfigInfo(screenconfig);
+		}
+		XCloseDisplay(display);
+	}
+
+	if (Res == NULL || nbRes == 0) {
+		// We failed to get a handle to the display, so fill in some defaults.
+		GfOut("Failed to initialize resolutions for display '%s'", XDisplayName(displayname));
+		nbRes = 8;
+		Res = (char **) malloc(sizeof(char *)*nbRes);
+		Res[0] = strdup("640x480");
+		Res[1] = strdup("800x600");
+		Res[2] = strdup("1024x768");
+		Res[3] = strdup("1152x864");
+		Res[4] = strdup("1200x960");
+		Res[5] = strdup("1280x1024");
+		Res[6] = strdup("1600x1200");
+		Res[7] = strdup("320x200");
+	}
+#endif // USE_RANDR_EXT
 }
 
 static void Reshape(int width, int height)
@@ -108,6 +241,7 @@ void GfScrInit(int argc, char *argv[])
     int		winX, winY;
     void	*handle;
     char	*fscr;
+	char	*vinit;
     int		fullscreen;
     int		maxfreq;
     int		i, depth;
@@ -125,129 +259,150 @@ void GfScrInit(int argc, char *argv[])
     GfScrCenX = xw / 2;
     GfScrCenY = yw / 2;
 
-    glutInit(&argc, argv);
-
-	// Try to get "best" videomode, z-buffer >= 24bit, visual with alpha channel,
-	// antialiasing support.
-
-	int visualDepthBits = 24;
-	bool visualSupportsMultisample = true;
-	bool visualSupportsAlpha = true;
-
-	glutInitDisplayString("rgba double depth>=24 samples alpha");
-
-	if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
-		// Failed, try without antialiasing support.
-		visualDepthBits = 24;
-		visualSupportsMultisample = false;
-		visualSupportsAlpha = true;
-		glutInitDisplayString("rgba double depth>=24 alpha");
-	}
-
-	if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
-		// Failed, try without alpha channel.
-		visualDepthBits = 24;
-		visualSupportsMultisample = true;
-		visualSupportsAlpha = false;
-		glutInitDisplayString("rgb double depth>=24 samples");
-	}
-
-	if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
-		// Failed, try without antialiasing and alpha support.
-		visualDepthBits = 24;
-		visualSupportsMultisample = false;
-		visualSupportsAlpha = false;
-		glutInitDisplayString("rgb double depth>=24");
-	}
-
-	if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
-		// Failed, try without 24 bit z-Buffer and without antialiasing.
-		visualDepthBits = 16;
-		visualSupportsMultisample = false;
-		visualSupportsAlpha = true;
-		glutInitDisplayString("rgba double depth>=16 alpha");
-	}
-
-	if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
-		// Failed, try without 24 bit z-Buffer, without antialiasing and without alpha.
-		visualDepthBits = 16;
-		visualSupportsMultisample = false;
-		visualSupportsAlpha = false;
-		glutInitDisplayString("rgb double depth>=16");
-	}
-
-	printf("Visual Properties Report\n");
-	printf("------------------------\n");
-
-	if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
-		// All failed.
-		printf("The minimum display requirements are not fulfilled.\n");
-		printf("We need a double buffered RGBA visual with a 16 bit depth buffer at least.\n");
-	} else {
-		// We have got a mode, report the properties.
-		printf("z-buffer depth: %d\n", visualDepthBits);
-		printf("alpha channel : %s\n", visualSupportsAlpha ? "enabled" : "disabled");
-		printf("antialiasing  : %s\n", visualSupportsMultisample ? "enabled" : "disabled");
-		if (visualDepthBits < 24) {
-			// Show a hint if the z-buffer depth is not optimal.
-			printf("The z-buffer resolution is below 24 bit, you will experience rendering\n");
-			printf("artefacts. Try to improve the setup of your graphics board or look\n");
-			printf("for an alternate driver.\n");
-		}
-	}
-
-    fscr = GfParmGetStr(handle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, GFSCR_VAL_NO);
-    fullscreen = 0;
-    if (strcmp(fscr, GFSCR_VAL_YES) == 0) {
+	// The fullscreen hack must be run before glutInit, such that glut gets the right screen size, etc.
+	fscr = GfParmGetStr(handle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, GFSCR_VAL_NO);
+	fullscreen = 0;
 #if !defined(FREEGLUT) && !defined(WIN32)
-    /* Resize the screen */
-	GfOut ("Freeglut not detected...\n");
-	for (i = maxfreq; i > 59; i--) {
-	    sprintf(buf, "%dx%d:%d@%d", winX, winY, depth, i);
-	    GfOut("Trying %s mode\n", buf);
-	    fglutGameModeString(buf);
-	    if (fglutEnterGameMode()) {
-		GfOut("OK done for %s\n", buf);
-		usedFG = 1;
-		break;
-	    }
+	if (strcmp(fscr, GFSCR_VAL_YES) == 0) {	// Resize the screen
+		GfOut ("Freeglut not detected...\n");
+		for (i = maxfreq; i > 59; i--) {
+			sprintf(buf, "%dx%d:%d@%d", winX, winY, depth, i);
+			GfOut("Trying %s mode\n", buf);
+			fglutGameModeString(buf);
+			if (fglutEnterGameMode()) {
+				GfOut("OK done for %s\n", buf);
+				usedFG = 1;
+				break;
+			}
+		}
 	}
 #endif
-	for (i = maxfreq; i > 59; i--) {
-	    sprintf(buf, "%dx%d:%d@%d", winX, winY, depth, i);
-	    glutGameModeString(buf);
-	    GfOut("2 - Trying %s mode\n", buf);
-	    if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) {
-		GfOut("2- %s mode Possible\n", buf);
-		glutEnterGameMode();
-		if (glutGameModeGet(GLUT_GAME_MODE_DISPLAY_CHANGED)) {
-		    GfOut("Use GameMode %s\n", buf);
-		    usedGM = 1;
-		    fullscreen = 1;
-		    break;
-		} else {
-		    glutLeaveGameMode();
+
+	vinit = GfParmGetStr(handle, GFSCR_SECT_PROP, GFSCR_ATT_VINIT, GFSCR_VAL_VINIT_COMPATIBLE);
+
+    glutInit(&argc, argv);
+
+	// Depending on "video mode init" setting try to get the best mode or try to get a mode in a safe way...
+	// This is a workaround for driver/glut/glx bug, which lie about the capabilites of the visual.
+
+	if (strcmp(vinit, GFSCR_VAL_VINIT_BEST) == 0) {
+
+		// Try to get "best" videomode, z-buffer >= 24bit, visual with alpha channel,
+		// antialiasing support.
+
+		int visualDepthBits = 24;
+		bool visualSupportsMultisample = true;
+		bool visualSupportsAlpha = true;
+
+		glutInitDisplayString("rgba double depth>=24 samples alpha");
+
+		if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
+			// Failed, try without antialiasing support.
+			visualDepthBits = 24;
+			visualSupportsMultisample = false;
+			visualSupportsAlpha = true;
+			glutInitDisplayString("rgba double depth>=24 alpha");
 		}
-	    }
-	}
-    }
 
-    if (!fullscreen) {
+		if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
+			// Failed, try without alpha channel.
+			visualDepthBits = 24;
+			visualSupportsMultisample = true;
+			visualSupportsAlpha = false;
+			glutInitDisplayString("rgb double depth>=24 samples");
+		}
+
+		if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
+			// Failed, try without antialiasing and alpha support.
+			visualDepthBits = 24;
+			visualSupportsMultisample = false;
+			visualSupportsAlpha = false;
+			glutInitDisplayString("rgb double depth>=24");
+		}
+
+		if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
+			// Failed, try without 24 bit z-Buffer and without antialiasing.
+			visualDepthBits = 16;
+			visualSupportsMultisample = false;
+			visualSupportsAlpha = true;
+			glutInitDisplayString("rgba double depth>=16 alpha");
+		}
+
+		if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
+			// Failed, try without 24 bit z-Buffer, without antialiasing and without alpha.
+			visualDepthBits = 16;
+			visualSupportsMultisample = false;
+			visualSupportsAlpha = false;
+			glutInitDisplayString("rgb double depth>=16");
+		}
+
+		printf("Visual Properties Report\n");
+		printf("------------------------\n");
+
+		if (!glutGet(GLUT_DISPLAY_MODE_POSSIBLE)) {
+			// All failed.
+			printf("The minimum display requirements are not fulfilled.\n");
+			printf("We need a double buffered RGB visual with a 16 bit depth buffer at least.\n");
+			// Try fallback as last resort.
+			printf("Trying generic initialization, fallback.\n");
+			glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
+		} else {
+			// We have got a mode, report the properties.
+			printf("z-buffer depth: %d (%s)\n", visualDepthBits, visualDepthBits < 24 ? "bad" : "good");
+			printf("multisampling : %s\n", visualSupportsMultisample ? "available" : "no");
+			printf("alpha bits    : %s\n", visualSupportsAlpha ? "available" : "no");
+			if (visualDepthBits < 24) {
+				// Show a hint if the z-buffer depth is not optimal.
+				printf("The z-buffer resolution is below 24 bit, you will experience rendering\n");
+				printf("artefacts. Try to improve the setup of your graphics board or look\n");
+				printf("for an alternate driver.\n");
+			}
+		}
+	} else {
+		// Compatibility mode.
+		glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
+		printf("Visual Properties Report\n");
+		printf("------------------------\n");
+		printf("Compatibility mode, properties unknown.\n");
+	}
+
+
+	if (strcmp(fscr, GFSCR_VAL_YES) == 0) {
+		for (i = maxfreq; i > 59; i--) {
+			sprintf(buf, "%dx%d:%d@%d", winX, winY, depth, i);
+			glutGameModeString(buf);
+			GfOut("2 - Trying %s mode\n", buf);
+			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) {
+				GfOut("2- %s mode Possible\n", buf);
+				glutEnterGameMode();
+				if (glutGameModeGet(GLUT_GAME_MODE_DISPLAY_CHANGED)) {
+					GfOut("Use GameMode %s\n", buf);
+					usedGM = 1;
+					fullscreen = 1;
+					break;
+				} else {
+					glutLeaveGameMode();
+				}
+			}
+		}
+	}
+
+	if (!fullscreen) {
 	/* Give an initial size and position so user doesn't have to place window */
-	glutInitWindowPosition(0, 0);
-	glutInitWindowSize(winX, winY);
-	Window = glutCreateWindow(argv[0]);
-	if (!Window) {
-	    printf("Error, couldn't open window\n");
-	    GfScrShutdown();
-	    exit(1);
+		glutInitWindowPosition(0, 0);
+		glutInitWindowSize(winX, winY);
+		Window = glutCreateWindow(argv[0]);
+		if (!Window) {
+			printf("Error, couldn't open window\n");
+			GfScrShutdown();
+			exit(1);
+		}
 	}
-    }
 
-    if ((strcmp(fscr, GFSCR_VAL_YES) == 0) && (!fullscreen)) {
-	/* glutVideoResize(0, 0, winX, winY); */
-	glutFullScreen();
-    }
+	if ((strcmp(fscr, GFSCR_VAL_YES) == 0) && (!fullscreen)) {
+		/* glutVideoResize(0, 0, winX, winY); */
+		glutFullScreen();
+	}
 
     GfParmReleaseHandle(handle);
 
@@ -268,6 +423,14 @@ void GfScrShutdown(void)
 	fglutLeaveGameMode();
     }
 #endif
+
+#ifdef USE_RANDR_EXT
+	int i;
+	for (i = 0; i < nbRes; i++) {
+		free(Res[i]);
+	}
+	free(Res);
+#endif // USE_RANDR_EXT
 }
 
 
@@ -290,24 +453,26 @@ void GfScrGetSize(int *scrw, int *scrh, int *vieww, int *viewh)
 static void
 saveParams(void)
 {
-    int x, y, bpp;
+	int x, y, bpp;
 
-    sscanf(Res[curRes], "%dx%d", &x, &y);
-    sscanf(Depth[curDepth], "%d", &bpp);
+	sscanf(Res[curRes], "%dx%d", &x, &y);
+	sscanf(Depthlist[curDepth], "%d", &bpp);
 
-    GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_X, (char*)NULL, x);
-    GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_Y, (char*)NULL, y);
-    GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_WIN_X, (char*)NULL, x);
-    GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_WIN_Y, (char*)NULL, y);
-    GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_BPP, (char*)NULL, bpp);
-    GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_MAXREFRESH, (char*)NULL, curMaxFreq);
+	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_X, (char*)NULL, x);
+	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_Y, (char*)NULL, y);
+	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_WIN_X, (char*)NULL, x);
+	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_WIN_Y, (char*)NULL, y);
+	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_BPP, (char*)NULL, bpp);
+	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_MAXREFRESH, (char*)NULL, curMaxFreq);
 
-    if (curMode == 0) {
-	GfParmSetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, "yes");
-    } else {
-	GfParmSetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, "no");
-    }
-    GfParmWriteFile(NULL, paramHdle, "Screen");
+	GfParmSetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_VINIT, VInit[curVInit]);
+
+	if (curMode == 0) {
+		GfParmSetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, "yes");
+	} else {
+		GfParmSetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, "no");
+	}
+	GfParmWriteFile(NULL, paramHdle, "Screen");
 }
 
 
@@ -392,19 +557,19 @@ static void
 updateLabelText(void)
 {
     GfuiLabelSetText (scrHandle, ResLabelId, Res[curRes]);
-    GfuiLabelSetText (scrHandle, DepthLabelId, Depth[curDepth]);
+    GfuiLabelSetText (scrHandle, DepthLabelId, Depthlist[curDepth]);
     GfuiLabelSetText (scrHandle, ModeLabelId, Mode[curMode]);
-#if WIN32
+#ifdef WIN32
     sprintf(buf, "%d", curMaxFreq);
     GfuiEditboxSetString(scrHandle, MaxFreqId, buf);
 #endif
+	GfuiLabelSetText (scrHandle, VInitLabelId, VInit[curVInit]);
 }
 
 static void
 ResPrevNext(void *vdelta)
 {
     long delta = (long)vdelta;
-
     curRes += (int)delta;
     if (curRes < 0) {
 	curRes = nbRes - 1;
@@ -448,42 +613,69 @@ ModePrevNext(void *vdelta)
     updateLabelText();
 }
 
+
+static void
+VInitPrevNext(void *vdelta)
+{
+	long delta = (long)vdelta;
+
+	curVInit += (int)delta;
+	if (curVInit < 0) {
+		curVInit = nbVInit - 1;
+	} else {
+		if (curVInit >= nbVInit) {
+			curVInit = 0;
+		}
+	}
+	updateLabelText();
+}
+
+
 static void
 initFromConf(void)
 {
-    int x, y, bpp;
-    int i;
+	int x, y, bpp;
+	int i;
 
-    x = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_X, NULL, 640);
-    y = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_Y, NULL, 480);
+	x = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_X, NULL, 640);
+	y = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_Y, NULL, 480);
 
-    sprintf(buf, "%dx%d", x, y);
-    for (i = 0; i < nbRes; i++) {
-	if (!strcmp(buf, Res[i])) {
-	    curRes = i;
-	    break;
+	sprintf(buf, "%dx%d", x, y);
+	for (i = 0; i < nbRes; i++) {
+		if (!strcmp(buf, Res[i])) {
+			curRes = i;
+			break;
+		}
 	}
-    }
 
-    if (!strcmp("yes", GfParmGetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, "yes"))) {
-	curMode = 0;
-    } else {
-	curMode = 1;
-    }
-
-    bpp = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_BPP, NULL, 24);
-    sprintf(buf, "%d", bpp);
-    for (i = 0; i < nbDepth; i++) {
-	if (!strcmp(buf, Depth[i])) {
-	    curDepth = i;
-	    break;
+	if (!strcmp("yes", GfParmGetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, "yes"))) {
+		curMode = 0;
+	} else {
+		curMode = 1;
 	}
-    }
 
-    curMaxFreq = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_MAXREFRESH, NULL, curMaxFreq);
+	curVInit = 0;
+	char *tmp = GfParmGetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_VINIT, GFSCR_VAL_VINIT_COMPATIBLE);
+	for (i = 0; i < nbVInit; i++) {
+		if (strcmp(VInit[i], tmp) == 0) {
+			curVInit = i;
+			break;
+		}
+	}
+
+	bpp = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_BPP, NULL, 24);
+	sprintf(buf, "%d", bpp);
+	for (i = 0; i < nbDepth; i++) {
+		if (!strcmp(buf, Depthlist[i])) {
+			curDepth = i;
+			break;
+		}
+	}
+
+	curMaxFreq = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_MAXREFRESH, NULL, curMaxFreq);
 }
 
-#if WIN32
+#ifdef WIN32
 static void
 ChangeMaxFreq(void * /* dummy */)
 {
@@ -512,7 +704,11 @@ void *
 GfScrMenuInit(void *precMenu)
 {
     int		y, x1, x2;
-    
+#ifndef WIN32
+	const int yoffset1 = 30, yoffset2 = 60;
+#else // WIN32
+	const int yoffset1 = 30, yoffset2 = 40;
+#endif // WIN32
     sprintf(buf, "%s%s", GetLocalDir(), GFSCR_CONF_FILE);
     paramHdle = GfParmReadFile(buf, GFPARM_RMODE_STD | GFPARM_RMODE_CREAT);
 
@@ -531,7 +727,7 @@ GfScrMenuInit(void *precMenu)
 		    320, y, GFUI_ALIGN_HC_VB,
 		    0);   
 
-    y -= 30;
+    y -= yoffset1; //30;
     GfuiGrButtonCreate(scrHandle,
 		       "data/img/arrow-left.png",
 		       "data/img/arrow-left.png",
@@ -559,13 +755,13 @@ GfScrMenuInit(void *precMenu)
 		       NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
     GfuiAddSKey(scrHandle, GLUT_KEY_RIGHT, "Next Resolution", (void*)1, ResPrevNext, NULL);
 
-    y -= 60;
+    y -= yoffset2; //60;
     GfuiLabelCreate(scrHandle,
 		    "Color Depth",
 		    GFUI_FONT_LARGE,
 		    320, y, GFUI_ALIGN_HC_VB,
-		    0);   
-    y -= 30;
+		    0);
+    y -= yoffset1; //30;
     GfuiGrButtonCreate(scrHandle,
 		       "data/img/arrow-left.png",
 		       "data/img/arrow-left.png",
@@ -591,13 +787,14 @@ GfScrMenuInit(void *precMenu)
 		       (void*)1, DepthPrevNext,
 		       NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
 
-    y -= 60;
+    y -= yoffset2; //60;
     GfuiLabelCreate(scrHandle,
 		    "Display Mode",
 		    GFUI_FONT_LARGE,
 		    320, y, GFUI_ALIGN_HC_VB,
-		    0);   
-    y -= 30;
+		    0);
+
+    y -= yoffset1; //30;
     GfuiGrButtonCreate(scrHandle,
 		       "data/img/arrow-left.png",
 		       "data/img/arrow-left.png",
@@ -624,18 +821,51 @@ GfScrMenuInit(void *precMenu)
 		       NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
     GfuiAddKey(scrHandle, 'f', "Display Mode", (void*)1, ModePrevNext, NULL);
 
-#if WIN32
-    y -= 60;
+#ifdef WIN32
+    y -= yoffset2; //60;
     GfuiLabelCreate(scrHandle,
 		    "Max Frequency",
 		    GFUI_FONT_LARGE,
 		    320, y, GFUI_ALIGN_HC_VB,
-		    0);   
-    y -= 30;
+		    0);
+    y -= yoffset1; //30;
     MaxFreqId = GfuiEditboxCreate(scrHandle, "", GFUI_FONT_MEDIUM_C,
 				   275, y, 0, 8, NULL, (tfuiCallback)NULL, ChangeMaxFreq);
 #endif
 
+	y -= yoffset2; //60;
+    GfuiLabelCreate(scrHandle,
+		    "Video Mode Initialization",
+		    GFUI_FONT_LARGE,
+		    320, y, GFUI_ALIGN_HC_VB,
+		    0);
+    y -= yoffset1; //30;
+    GfuiGrButtonCreate(scrHandle,
+		       "data/img/arrow-left.png",
+		       "data/img/arrow-left.png",
+		       "data/img/arrow-left.png",
+		       "data/img/arrow-left-pushed.png",
+		       x1, y, GFUI_ALIGN_HC_VB, 0,
+		       (void*)-1, VInitPrevNext,
+		       NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
+
+    VInitLabelId = GfuiLabelCreate(scrHandle,
+				  "",
+				  GFUI_FONT_LARGE_C,
+				  320, y, GFUI_ALIGN_HC_VB,
+				  30);
+    GfuiLabelSetColor(scrHandle, VInitLabelId, LabelColor);
+
+    GfuiGrButtonCreate(scrHandle,
+		       "data/img/arrow-right.png",
+		       "data/img/arrow-right.png",
+		       "data/img/arrow-right.png",
+		       "data/img/arrow-right-pushed.png",
+		       x2, y, GFUI_ALIGN_HC_VB, 0,
+		       (void*)1, VInitPrevNext,
+		       NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
+
+
     GfuiAddKey(scrHandle, 13, "Apply Mode", NULL, GfScrReinit, NULL);
     GfuiButtonCreate(scrHandle, "Apply", GFUI_FONT_LARGE, 210, 40, 150, GFUI_ALIGN_HC_VB, GFUI_MOUSE_UP,
 		     NULL, GfScrReinit, NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
@@ -643,9 +873,13 @@ GfScrMenuInit(void *precMenu)
     GfuiAddKey(scrHandle, 27, "Cancel", precMenu, GfuiScreenActivate, NULL);
     GfuiButtonCreate(scrHandle, "Back", GFUI_FONT_LARGE, 430, 40, 150, GFUI_ALIGN_HC_VB, GFUI_MOUSE_UP,
 		     precMenu, GfuiScreenActivate, NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
- 
+
     return scrHandle;
 }
 
 
 
+int GfuiGlutExtensionSupported(char *str)
+{
+    return glutExtensionSupported(str);
+}
