@@ -4,7 +4,7 @@
     created              : Thu Aug 17 23:52:20 CEST 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: grboard.cpp,v 1.20 2004/02/06 23:08:53 torcs Exp $
+    version              : $Id$
 
  ***************************************************************************/
 
@@ -675,10 +675,10 @@ public:
 void
 cGrBoard::grDispCounterBoard2(tCarElt *car)
 {
-    int			index;
-    tgrCarInstrument	*curInst;
-    tdble		val;
-    char		buf[32];
+    int index;
+    tgrCarInstrument *curInst;
+    tdble val;
+    char buf[32];
 
     index = car->index;	/* current car's index */
     curInst = &(grCarInfo[index].instrument[0]);
@@ -731,22 +731,23 @@ cGrBoard::grDispCounterBoard2(tCarElt *car)
     val = curInst->minAngle + val * curInst->maxAngle;
 
     RELAXATION(val, curInst->prevVal, 30);
-    
+
     glPushMatrix();
     glTranslatef(curInst->needleXCenter, curInst->needleYCenter, 0);
     glRotatef(val, 0, 0, 1);
     glCallList(curInst->needleList);
     glPopMatrix();
 
-    if (curInst->digital) {
-	sprintf(buf, "%3d", abs((int)(car->_speed_x * 3.6)));
-	GfuiPrintString(buf, grBlue, GFUI_FONT_DIGIT,
+	if (curInst->digital) {
+		// Do not add "%3d" or something, because the digital font DOES NOT SUPPORT BLANKS!!!!
+		sprintf(buf, "%d", abs((int)(car->_speed_x * 3.6)));
+		GfuiPrintString(buf, grBlue, GFUI_FONT_DIGIT,
 			(int)curInst->digitXCenter, (int)(curInst->digitYCenter), GFUI_ALIGN_HC_VB);
-    }
-    
-    if (counterFlag == 1){
-	grDispMisc(car);
-    }
+	}
+
+	if (counterFlag == 1){
+		grDispMisc(car);
+	}
 }
 
 void
@@ -760,7 +761,10 @@ cGrBoard::initBoard(void)
 void
 cGrBoard::shutdown(void)
 {
-	delete trackMap;
+	if (trackMap != NULL) {
+		delete trackMap;
+		trackMap = NULL;
+	}
 }
 
 void
@@ -820,25 +824,28 @@ cGrBoard::grDispArcade(tCarElt *car, tSituation *s)
     grDispEngineLeds (car, Winx + Winw - XM, YM + dy + GfuiFontHeight (GFUI_FONT_BIG_C), ALIGN_RIGHT, 0);
 }
 
-void
-cGrBoard::refreshBoard(tSituation *s, float Fps, int forceArcade, tCarElt *curr)
+
+void cGrBoard::refreshBoard(tSituation *s, float Fps, int forceArcade, tCarElt *curr)
 {
-    if (arcadeFlag || forceArcade) {
-	grDispArcade(curr, s);
-    } else {
-	if (debugFlag)   grDispDebug(Fps, curr);
-	if (GFlag)       grDispGGraph(curr);
-	if (boardFlag)   grDispCarBoard(curr, s);
-	if (leaderFlag)	 grDispLeaderBoard(curr, s);
-	if (counterFlag) grDispCounterBoard2(curr);
-    }
-#ifndef NO_ALPHA_DISPLAY
+	if (arcadeFlag || forceArcade) {
+		grDispArcade(curr, s);
+	} else {
+		if (debugFlag) grDispDebug(Fps, curr);
+		if (GFlag) grDispGGraph(curr);
+		if (boardFlag) grDispCarBoard(curr, s);
+		if (leaderFlag)	grDispLeaderBoard(curr, s);
+		if (counterFlag) grDispCounterBoard2(curr);
+	}
+
 	trackMap->display(curr, s, Winx, Winy, Winw, Winh);
-#endif
 }
 
-void
-grInitBoardCar(tCarElt *car)
+// TODO: clean solution for cleanup.
+static ssgSimpleState* cleanup[1024];
+static int nstate = 0;
+
+
+void grInitBoardCar(tCarElt *car)
 {
     char		buf[4096];
     int			index;
@@ -849,9 +856,9 @@ grInitBoardCar(tCarElt *car)
     tgrCarInstrument	*curInst;
     tdble		xSz, ySz, xpos, ypos;
     tdble		needlexSz, needleySz;
-    
+
     ssgSetCurrentOptions ( &options ) ;
-    
+
     index = car->index;	/* current car's index */
     carInfo = &grCarInfo[index];
     handle = car->_carHandle;
@@ -865,6 +872,8 @@ grInitBoardCar(tCarElt *car)
     grFilePath = strdup(buf);
     curInst->texture = (ssgSimpleState*)grSsgLoadTexState(param);
     free(grFilePath);
+	cleanup[nstate] = curInst->texture;
+	nstate++;
 
     /* Load the intrument placement */
     xSz = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_XSZ, (char*)NULL, 128);
@@ -876,14 +885,14 @@ grInitBoardCar(tCarElt *car)
     curInst->needleXCenter = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_XCENTER, (char*)NULL, xSz / 2.0) + xpos;
     curInst->needleYCenter = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_YCENTER, (char*)NULL, ySz / 2.0) + ypos;
     curInst->digitXCenter = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_XDIGITCENTER, (char*)NULL, xSz / 2.0) + xpos;
-    curInst->digitYCenter = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_YDIGITCENTER, (char*)NULL, 16) + ypos; 
+    curInst->digitYCenter = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_YDIGITCENTER, (char*)NULL, 16) + ypos;
     curInst->minValue = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_MINVAL, (char*)NULL, 0);
     curInst->maxValue = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_MAXVAL, (char*)NULL, RPM2RADS(10000)) - curInst->minValue;
     curInst->minAngle = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_MINANG, "deg", 225);
     curInst->maxAngle = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_MAXANG, "deg", -45) - curInst->minAngle;
     curInst->monitored = &(car->_enginerpm);
     curInst->prevVal = curInst->minAngle;
-    
+
     curInst->CounterList = glGenLists(1);
     glNewList(curInst->CounterList, GL_COMPILE);
     glBegin(GL_TRIANGLE_STRIP);
@@ -913,13 +922,15 @@ grInitBoardCar(tCarElt *car)
 
     /* Speedometer */
     curInst = &(carInfo->instrument[1]);
-    
+
     /* Load the Speedometer texture */
     param = GfParmGetStr(handle, SECT_GROBJECTS, PRM_SPEEDO_TEX, "speed360.rgb");
     sprintf(buf, "drivers/%s/%d;drivers/%s;cars/%s;data/textures", car->_modName, car->_driverIndex, car->_modName, car->_carName);
     grFilePath = strdup(buf);
     curInst->texture = (ssgSimpleState*)grSsgLoadTexState(param);
     free(grFilePath);
+	cleanup[nstate] = curInst->texture;
+	nstate++;
 
     /* Load the intrument placement */
     xSz = GfParmGetNum(handle, SECT_GROBJECTS, PRM_SPEEDO_XSZ, (char*)NULL, 128);
@@ -969,3 +980,17 @@ grInitBoardCar(tCarElt *car)
     glEndList();
     
 }
+
+void grShutdownBoardCar(void)
+{
+	/*int i;
+	for (i = 0; i < nstate; i++) {
+		printf("%d\n", i);
+		if (cleanup[i]->getRef() > 0) {
+			ssgDeRefDelete(cleanup[i]);
+		} else {
+			delete cleanup[i];
+		}
+	}
+	nstate = 0;*/
+}
