@@ -4,7 +4,7 @@
     created              : Mon Aug 21 20:55:32 CEST 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: grcam.cpp,v 1.39 2004/01/16 20:13:55 torcs Exp $
+    version              : $Id$
 
  ***************************************************************************/
 
@@ -315,16 +315,28 @@ void cGrCarCamMirror::update(tCarElt *car, tSituation * /* s */)
 
 void cGrCarCamMirror::setViewport (int x, int y, int w, int h)
 {
-    vpx = x;
-    vpy = y;
-    vpw = GfNearestPow2 (w);
-    vph = GfNearestPow2 (h);
+	vpx = x;
+	vpy = y;
+	vpw = GfNearestPow2 (w);
+	vph = GfNearestPow2 (h);
 
-    if (viewCam) {
-	delete viewCam;
-    }
-    viewCam = new cGrOrthoCamera (screen, x,  x + w, y, y + h);
-    limitFov ();
+	// Create texture object.
+	glBindTexture (GL_TEXTURE_2D, tex);
+	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+	glReadBuffer (GL_BACK);
+
+	glCopyTexImage2D (GL_TEXTURE_2D,
+			0, // map level,
+			GL_RGB, // internal format,
+			vpx, vpy, vpw, vph,
+			0 ); // border
+
+	if (viewCam) {
+		delete viewCam;
+	}
+	viewCam = new cGrOrthoCamera (screen, x,  x + w, y, y + h);
+	limitFov ();
 }
 
 void cGrCarCamMirror::setPos (int x, int y, int w, int h)
@@ -342,7 +354,7 @@ void cGrCarCamMirror::setPos (int x, int y, int w, int h)
     dy = MAX (0, dy);
     dx = dx / (float)vpw / 2.0;
     dy = dy / (float)vph / 2.0;
-    
+
     tsu = 1.0 - dx;
     tsv = dy;
     teu = dx;
@@ -357,21 +369,25 @@ void cGrCarCamMirror::activateViewport (void)
 
 void cGrCarCamMirror::store (void)
 {
-    glBindTexture (GL_TEXTURE_2D, tex);
-    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-    glReadBuffer (GL_BACK);
-    glCopyTexImage2D (GL_TEXTURE_2D,
-		      0 /* map level */,
-		      GL_RGB /* internal format */,
-		      vpx, vpy, vpw, vph,
-		      0 /* border */ );
+	glBindTexture (GL_TEXTURE_2D, tex);
+	//glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	//glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+	glReadBuffer (GL_BACK);
+
+	/*glCopyTexImage2D (GL_TEXTURE_2D,
+			0, // map level,
+			GL_RGB, // internal format,
+			vpx, vpy, vpw, vph,
+			0 ); // border
+*/
+	// NVidia recommends to NOT use glCopyTexImage2D for performance reasons.
+	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, vpx, vpy, vpw, vph);
 }
 
 void cGrCarCamMirror::display (void)
 {
     viewCam->action ();
-    
+
     glBindTexture (GL_TEXTURE_2D, tex);
     glBegin(GL_TRIANGLE_STRIP);
     {
@@ -972,7 +988,7 @@ class cGrCarCamRoadFly : public cGrPerspCamera
 
 	    if (current != car->index) {
 		/* the target car changed */
-		zOffset = 40.0;
+		zOffset = 50.0;
 		current = car->index;
 	    } else {
 		zOffset = 0.0;
@@ -982,7 +998,7 @@ class cGrCarCamRoadFly : public cGrPerspCamera
 		timer = 500 + (int)(500.0*rand()/(RAND_MAX+1.0));
 		offset[0] = -0.5 + (rand()/(RAND_MAX+1.0));
 		offset[1] = -0.5 + (rand()/(RAND_MAX+1.0));
-		offset[2] = -5.0 + (30.0*rand()/(RAND_MAX+1.0)) + zOffset;
+		offset[2] = 30 + (30.0*rand()/(RAND_MAX+1.0)) + zOffset;
 		offset[0] = offset[0]*(offset[2]+1.0);
 		offset[1] = offset[1]*(offset[2]+1.0);
 		gain = 5.0;
@@ -1161,7 +1177,9 @@ class cGrCarCamRoadZoomTVD : public cGrCarCamRoadZoom
 	camEventInterval  = GfParmGetNum(grHandle, GR_SCT_TVDIR, GR_ATT_EVTINT, (char*)NULL, 1.0);
 	proximityThld     = GfParmGetNum(grHandle, GR_SCT_TVDIR, GR_ATT_PROXTHLD, (char*)NULL, 10.0);
     }
-    
+
+    ~cGrCarCamRoadZoomTVD() { free(schedView); }
+
     void update(tCarElt *car, tSituation *s) {
 	int	i, j;
 	int	curCar;
@@ -1626,12 +1644,15 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			  id,
 			  1,	/* drawCurr */
 			  1,	/* drawBG  */
-			  12.0,	/* fovy */
+			  //12.0,	/* fovy */
+			  67.5,	/* fovy */
 			  1.0,	/* fovymin */
 			  90.0,	/* fovymax */
-			  300.0,	/* distz */
+			  //300.0,	/* distz */
+			  200.0,	/* distz */
 			  0,		/* axis */
-			  200.0,	/* near */
+			  //200.0,	/* near */
+			  100.0,	/* near */
 			  1000.0 * fovFactor,/* far */
 			  500.0 * fovFactor,	/* fog */
 			  1000.0 * fovFactor	/* fog */
@@ -1644,10 +1665,12 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			  id,
 			  1,	/* drawCurr */
 			  1,	/* drawBG  */
-			  12.0,	/* fovy */
+			  //12.0,	/* fovy */
+			  67.5,	/* fovy */
 			  1.0,	/* fovymin */
 			  90.0,	/* fovymax */
-			  300.0,	/* distz */
+			  //300.0,	/* distz */
+			  250.0,	/* distz */
 			  1,		/* axis */
 			  200.0,	/* near */
 			  1000.0 * fovFactor,/* far */
@@ -1662,10 +1685,12 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			  id,
 			  1,	/* drawCurr */
 			  1,	/* drawBG  */
-			  12.0,	/* fovy */
+			  //12.0,	/* fovy */
+			  67.5,	/* fovy */
 			  1.0,	/* fovymin */
 			  90.0,	/* fovymax */
-			  300.0,	/* distz */
+			  //300.0,	/* distz */
+			  350.0,	/* distz */
 			  2,		/* axis */
 			  200.0,	/* near */
 			  1000.0 * fovFactor,/* far */
@@ -1680,10 +1705,12 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			  id,
 			  1,	/* drawCurr */
 			  1,	/* drawBG  */
-			  12.0,	/* fovy */
+			  //12.0,	/* fovy */
+			  67.5,	/* fovy */
 			  1.0,	/* fovymin */
 			  90.0,	/* fovymax */
-			  300.0,	/* distz */
+			  //300.0,	/* distz */
+			  400.0,	/* distz */
 			  3,		/* axis */
 			  200.0,	/* near */
 			  1000.0 * fovFactor,/* far */
@@ -1883,7 +1910,8 @@ grCamCreateSceneCameraList(class cGrScreen *myscreen, tGrCamHead *cams, tdble fo
 			       id,
 			       1,		/* drawCurr */
 			       1,		/* drawBG  */
-			       17.0,	/* fovy */
+			       //17.0,	/* fovy */
+			       67.5,	/* fovy */
 			       1.0,		/* fovymin */
 			       90.0,	/* fovymax */
 			       1.0,		/* near */
