@@ -1,10 +1,11 @@
+
 /***************************************************************************
 
     file                 : grcar.cpp
     created              : Mon Aug 21 18:24:02 CEST 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: grcar.cpp,v 1.32 2003/11/08 16:37:18 torcs Exp $
+    version              : $Id$
 
 ***************************************************************************/
 
@@ -387,11 +388,50 @@ grInitShadow(tCarElt *car)
     };
 
     grCarInfo[car->index].shadowBase = new ssgVtxTableShadow(GL_TRIANGLE_STRIP, shd_vtx, shd_nrm, shd_tex, shd_clr);
-    grMipMap = 0;
+	grMipMap = 0;
     grCarInfo[car->index].shadowBase->setState(grSsgLoadTexState(shdTexName));
-    grCarInfo[car->index].shadowCurr = (ssgVtxTableShadow *)grCarInfo[car->index].shadowBase->clone(SSG_CLONE_GEOMETRY);
+	grCarInfo[car->index].shadowCurr = (ssgVtxTableShadow *)grCarInfo[car->index].shadowBase->clone(SSG_CLONE_GEOMETRY);
     grCarInfo[car->index].shadowAnchor->addKid(grCarInfo[car->index].shadowCurr);
     ShadowAnchor->addKid(grCarInfo[car->index].shadowAnchor);
+    grCarInfo[car->index].shadowBase->ref();
+
+}
+
+void grPropagateDamage (ssgEntity* l, sgVec3 poc, sgVec3 force, int cnt) 
+{
+	//showEntityType (l);
+	if (l->isAKindOf (ssgTypeBranch())) {
+		
+		ssgBranch* br = (ssgBranch*) l;
+
+		for (int i = 0 ; i < br -> getNumKids () ; i++ ) {
+			ssgEntity* ln = br->getKid (i);
+			grPropagateDamage(ln, poc, force, cnt+1);
+		}
+	}
+	
+	if (l->isAKindOf (ssgTypeVtxTable())) {
+		sgVec3* v;
+		int Nv;
+		ssgVtxTable* vt = (ssgVtxTable*) l;
+		Nv = vt->getNumVertices();
+		vt->getVertexList ((void**) &v);
+		tdble sigma = sgLengthVec3 (force);
+		tdble invSigma = 5.0;
+		//		if (sigma < 0.1) 
+		//			invSigma = 10.0;
+		//		else
+		//			invSigma = 1.0/sigma;
+		for (int i=0; i<Nv; i++) {
+			tdble r =  sgDistanceSquaredVec3 (poc, v[i]);
+			tdble f = exp(-r*invSigma)*5.0;
+			v[i][0] += force[0]*f;
+			v[i][1] += force[1]*f;
+			// use sigma as a random number generator (!)
+			v[i][2] += (force[2]+0.02*sin(2.0*r + 10.0*sigma))*f;
+			//printf ("(%f %f %f)\n", v[i][0], v[i][1], v[i][2]);
+		}
+	}
 }
 
 
@@ -426,7 +466,7 @@ grInitCar(tCarElt *car)
 
     TRACE_GL("loadcar: start");
 
-    ssgSetCurrentOptions ( &options ) ;    
+    ssgSetCurrentOptions ( &options ) ;
 
     grCarIndex = index = car->index;	/* current car's index */
     handle = car->_carHandle;
@@ -476,11 +516,14 @@ grInitCar(tCarElt *car)
     lg += sprintf(grFilePath + lg, "cars/%s", car->_carName);
 
     param = GfParmGetStr(handle, SECT_GROBJECTS, PRM_WHEEL_TEXTURE, "");
-    if (strlen(param) != 0) {
-	grGammaValue = 1.8;
-	grMipMap = 0;
-	grCarInfo[index].wheelTexture = grSsgLoadTexState(param);
-    }
+	if (strlen(param) != 0) {
+		grGammaValue = 1.8;
+		grMipMap = 0;
+		grCarInfo[index].wheelTexture = grSsgLoadTexState(param);
+		/*if (grCarInfo[index].wheelTexture->getRef() > 0) {
+			grCarInfo[index].wheelTexture->deRef();
+		}*/
+	}
     
     grCarInfo[index].envSelector = (ssgStateSelector*)grEnvSelector->clone();
     grCarInfo[index].envSelector->ref();
@@ -504,7 +547,7 @@ grInitCar(tCarElt *car)
     ssgBranch *carBody = new ssgBranch;
     DBG_SET_NAME(carBody, "LOD", index, 0);
     LODSel->addKid(carBody);
-    
+
     /* The car's model is under cars/<model> */
     sprintf(buf, "cars/%s", car->_carName);
     ssgModelPath(buf);
@@ -520,18 +563,22 @@ grInitCar(tCarElt *car)
     grCarInfo[index].LODThreshold[selIndex] = GfParmGetNum(handle, path, PRM_THRESHOLD, NULL, 0.0);
     /*carEntity = ssgLoad(param);*/
     carEntity = grssgCarLoadAC3D(param, NULL, index);
+	grCarInfo[index].carEntity = carEntity;
     /* Set a selector on the driver */
     ssgBranch *b = (ssgBranch *)carEntity->getByName( "DRIVER" );
     grCarInfo[index].driverSelector = new ssgSelector;
-    if (b) {
-	ssgBranch *bp = b->getParent(0);
-	bp->addKid(grCarInfo[index].driverSelector);
-	grCarInfo[index].driverSelector->addKid(b);
-	bp->removeKid(b);
-	grCarInfo[index].driverSelector->select(1);
-    }
-    
- 
+	if (b) {
+		ssgBranch *bp = b->getParent(0);
+		bp->addKid(grCarInfo[index].driverSelector);
+		grCarInfo[index].driverSelector->addKid(b);
+		bp->removeKid(b);
+		grCarInfo[index].driverSelector->select(1);
+		grCarInfo[index].driverSelectorinsg = true;
+    } else {
+		grCarInfo[index].driverSelectorinsg = false;
+	}
+
+
     DBG_SET_NAME(carEntity, "Body", index, -1);
     carBody->addKid(carEntity);
     /* add wheels */
@@ -636,7 +683,16 @@ grDrawCar(tCarElt *car, tCarElt *curCar, int dispCarFlag, int dispDrvFlag, doubl
     TRACE_GL("cggrDrawCar: start");
 
     index = car->index;
-  
+	if (car->priv.collision_state.collision_count > 0) {
+		tCollisionState* collision_state = &car->priv.collision_state;
+		grPropagateDamage (grCarInfo[index].carEntity, collision_state->pos, collision_state->force, 0);
+		//printf("%d: ",collision_state->collision_count );
+		//for (int i=0; i<3; i++) {
+		//printf ("(%f %f) ", collision_state->pos[i], collision_state->force[i]);
+		//}
+		//printf("\n");
+		collision_state->collision_count = 0;
+	}
     grCarInfo[index].distFromStart=grGetDistToStart(car);
     grCarInfo[index].envAngle=RAD2DEG(car->_yaw);
 
