@@ -4,7 +4,7 @@
     created              : Fri Aug 29 00:58:00 CEST 2003
     copyright            : (C) 2003 by Bernhard Wymann
     email                : berniw@bluewin.ch
-    version              : $Id: grtrackmap.cpp,v 1.6 2003/11/07 14:17:42 berniw Exp $
+    version              : $Id$
 
  ***************************************************************************/
 
@@ -25,6 +25,7 @@
 */
 
 #include "grtrackmap.h"
+#include <tgfclient.h>
 
 // The resolution in [m] to analyse turns.
 const float cGrTrackMap::RESOLUTION = 5.0;
@@ -91,7 +92,16 @@ cGrTrackMap::cGrTrackMap()
 // the build host, include support for GL_ARB_texture_compression.
 #ifdef GL_ARB_texture_compression
 		// Query if the extension is avaiable at the runtime system (true, if > 0).
-		int compressARB = glutExtensionSupported("GL_ARB_texture_compression");
+		int compressARB = GfuiGlutExtensionSupported("GL_ARB_texture_compression");
+		// Check if at least one internal format is vailable. This is a workaround for
+		// driver problems and not a bugfix. According to the specification OpenGL should
+		// choose an uncompressed alternate format if it can't provide the requested
+		// compressed one... but it does not on all cards/drivers.
+		int numformats;
+		glGetIntegerv(GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB, &numformats);
+		if (numformats == 0) {
+			compressARB = 0;
+		}
 #endif //GL_ARB_texture_compression
 
 
@@ -170,6 +180,13 @@ cGrTrackMap::cGrTrackMap()
 		}
 		texturesize >>= 1;
 
+		// Get maximum OpenGL texture size and reduce texturesize if necessary.
+		int maxOpenGLtexturesize;
+		glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxOpenGLtexturesize);
+		if (texturesize > maxOpenGLtexturesize) {
+			texturesize = maxOpenGLtexturesize;
+		}
+
 		// Compute an estimate of the overall width and height of the track in [m].
 		track_width = track_max_x - track_min_x;
 		track_height = track_max_y - track_min_y;
