@@ -3,7 +3,7 @@
     file                 : grvtxtable.cpp
     created              : Fri Mar 22 23:16:44 CET 2002
     copyright            : (C) 2001 by Christophe Guionneau
-    version              : $Id: grvtxtable.cpp,v 1.10 2003/10/19 18:59:38 torcs Exp $
+    version              : $Id$
 
  ***************************************************************************/
 
@@ -120,7 +120,7 @@ grVtxTable::grVtxTable (GLenum ty, ssgVertexArray   *vl,
 			 ssgTexCoordArray *tl3,
 			 int _numMapLevel,
 			 int _mapLevel,
-			 ssgColourArray   *cl, 
+			 ssgColourArray   *cl,
 			 int _indexCar) : ssgVtxTable(ty, vl, nl, tl, cl)
 {
   type = ssgTypeVtxTable ();
@@ -174,9 +174,9 @@ grVtxTable::grVtxTable (GLenum ty, ssgVertexArray   *vl,
 
 grVtxTable::~grVtxTable ()
 {
-  /*  ssgDeRefDelete (vertices );
+    /*ssgDeRefDelete (vertices );
       ssgDeRefDelete (normals  );
-      ssgDeRefDelete (colours  ); */
+      ssgDeRefDelete (colours  );*/
       ssgDeRefDelete (texcoords1);
       ssgDeRefDelete (texcoords2);
       ssgDeRefDelete (texcoords3);
@@ -186,6 +186,10 @@ grVtxTable::~grVtxTable ()
 	  ssgDeRefDelete (stripes     );
 	}
 
+	ssgDeRefDelete(state1);
+	ssgDeRefDelete(state2);
+	ssgDeRefDelete(state3);
+
 } 
 
 void grVtxTable::setState1 (ssgState *st)
@@ -347,6 +351,11 @@ void grVtxTable::draw_geometry_for_a_car ()
   ttx=grCarInfo[indexCar].distFromStart/50;
   sgMakeTransMat4 (mat, ttx, tty, ttz);
   glMultMatrixf((float *)mat);
+/*	axis[0]=0;
+	axis[1]=1;
+	axis[2]=0;
+  sgMakeRotMat4(mat, -grCarInfo[indexCar].envAngle, axis);
+  glMultMatrixf((float *)mat);*/
   glMatrixMode(GL_MODELVIEW);
 
   int num_colours   = getNumColours   ();
@@ -561,9 +570,16 @@ void grVtxTable::draw_geometry_for_a_car_array ()
   glEnable (GL_TEXTURE_2D);
   glMatrixMode(GL_TEXTURE);
   glLoadIdentity();
+
   ttx=grCarInfo[indexCar].distFromStart/50;
   sgMakeTransMat4 (mat, ttx, tty, ttz);
+/*  glMultMatrixf((float *)mat);
+	axis[0]=0;
+	axis[1]=1;
+	axis[2]=0;
+  sgMakeRotMat4(mat, -grCarInfo[indexCar].envAngle, axis);*/
   glMultMatrixf((float *)mat);
+
   glMatrixMode(GL_MODELVIEW);
 
   glActiveTextureARB (GL_TEXTURE0_ARB);
