@@ -4,7 +4,7 @@
     created     : Sun Dec 15 11:12:56 CET 2002
     copyright   : (C) 2002 by Eric Espi                        
     email       : eric.espie@torcs.org   
-    version     : $Id: categories.cpp,v 1.4 2004/01/15 18:23:22 torcs Exp $                                  
+    version     : $Id: categories.cpp,v 1.5 2005/02/01 18:54:02 berniw Exp $                                  
 
  ***************************************************************************/
 
@@ -20,13 +20,13 @@
 /** @file   
     		
     @author	<a href=mailto:eric.espie@torcs.org>Eric Espie</a>
-    @version	$Id: categories.cpp,v 1.4 2004/01/15 18:23:22 torcs Exp $
+    @version	$Id: categories.cpp,v 1.5 2005/02/01 18:54:02 berniw Exp $
 */
 
 #include <stdio.h>
 #include "sim.h"
 
 
-tdble simDammageFactor[] = {0.0, 0.5, 0.8, 1.0, 1.0};
+tdble simDammageFactor[] = {0.0f, 0.5f, 0.8f, 1.0f, 1.0f};
 
-tdble simSkidFactor[] = {0.40, 0.35, 0.3, 0.0, 0.0};
+tdble simSkidFactor[] = {0.40f, 0.35f, 0.3f, 0.0f, 0.0f};
